@@ -14,7 +14,7 @@ use crate::{
     ebpf,
     error::{EbpfError, UserDefinedError},
     jit::JitProgram,
-    vm::{Config, Executable, InstructionMeter, SyscallRegistry},
+    vm::{Config, InstructionMeter, SyscallRegistry},
 };
 use byteorder::{ByteOrder, LittleEndian};
 use goblin::{
@@ -122,9 +122,15 @@ pub fn register_bpf_function(
     bpf_functions: &mut BTreeMap<u32, (usize, String)>,
     pc: usize,
     name: &str,
+    enable_symbol_and_section_labels: bool,
 ) -> Result<u32, ElfError> {
     let hash = hash_bpf_function(pc, name);
-    if let Some(entry) = bpf_functions.insert(hash, (pc, name.to_string())) {
+    let name = if enable_symbol_and_section_labels {
+        name.to_string()
+    } else {
+        String::default()
+    };
+    if let Some(entry) = bpf_functions.insert(hash, (pc, name)) {
         if entry.0 != pc {
             return Err(ElfError::SymbolHashCollision(hash));
         }
@@ -209,7 +215,7 @@ struct SectionInfo {
 
 /// Elf loader/relocator
 #[derive(Debug, PartialEq)]
-pub struct EBpfElf<E: UserDefinedError, I: InstructionMeter> {
+pub struct Executable<E: UserDefinedError, I: InstructionMeter> {
     /// Configuration settings
     config: Config,
     /// Loaded and executable elf
@@ -228,14 +234,14 @@ pub struct EBpfElf<E: UserDefinedError, I: InstructionMeter> {
     compiled_program: Option<JitProgram<E, I>>,
 }
 
-impl<E: UserDefinedError, I: InstructionMeter> Executable<E, I> for EBpfElf<E, I> {
+impl<E: UserDefinedError, I: InstructionMeter> Executable<E, I> {
     /// Get the configuration settings
-    fn get_config(&self) -> &Config {
+    pub fn get_config(&self) -> &Config {
         &self.config
     }
 
     /// Get the .text section virtual address and bytes
-    fn get_text_bytes(&self) -> (u64, &[u8]) {
+    pub fn get_text_bytes(&self) -> (u64, &[u8]) {
         let offset = (self.text_section_info.vaddr - ebpf::MM_PROGRAM_START) as usize;
         (
             self.text_section_info.vaddr,
@@ -244,12 +250,12 @@ impl<E: UserDefinedError, I: InstructionMeter> Executable<E, I> for EBpfElf<E, I
     }
 
     /// Get the concatenated read-only sections (including the text section)
-    fn get_ro_section(&self) -> &[u8] {
+    pub fn get_ro_section(&self) -> &[u8] {
         self.ro_section.as_slice()
     }
 
     /// Get the entry point offset into the text section
-    fn get_entrypoint_instruction_offset(&self) -> Result<usize, EbpfError<E>> {
+    pub fn get_entrypoint_instruction_offset(&self) -> Result<usize, EbpfError<E>> {
         self.bpf_functions
             .get(&ebpf::hash_symbol_name(b"entrypoint"))
             .map(|(pc, _name)| *pc)
@@ -257,28 +263,28 @@ impl<E: UserDefinedError, I: InstructionMeter> Executable<E, I> for EBpfElf<E, I
     }
 
     /// Get a symbol's instruction offset
-    fn lookup_bpf_function(&self, hash: u32) -> Option<usize> {
+    pub fn lookup_bpf_function(&self, hash: u32) -> Option<usize> {
         self.bpf_functions.get(&hash).map(|(pc, _name)| *pc)
     }
 
     /// Get the syscall registry
-    fn get_syscall_registry(&self) -> &SyscallRegistry {
+    pub fn get_syscall_registry(&self) -> &SyscallRegistry {
         &self.syscall_registry
     }
 
     /// Get the JIT compiled program
-    fn get_compiled_program(&self) -> Option<&JitProgram<E, I>> {
+    pub fn get_compiled_program(&self) -> Option<&JitProgram<E, I>> {
         self.compiled_program.as_ref()
     }
 
     /// JIT compile the executable
-    fn jit_compile(&mut self) -> Result<(), EbpfError<E>> {
+    pub fn jit_compile(&mut self) -> Result<(), EbpfError<E>> {
         self.compiled_program = Some(JitProgram::<E, I>::new(self)?);
         Ok(())
     }
 
     /// Report information on a symbol that failed to be resolved
-    fn report_unresolved_symbol(&self, insn_offset: usize) -> Result<u64, EbpfError<E>> {
+    pub fn report_unresolved_symbol(&self, insn_offset: usize) -> Result<u64, EbpfError<E>> {
         let file_offset = insn_offset
             .saturating_mul(ebpf::INSN_SIZE)
             .saturating_add(self.text_section_info.offset_range.start as usize);
@@ -312,7 +318,7 @@ impl<E: UserDefinedError, I: InstructionMeter> Executable<E, I> for EBpfElf<E, I
     }
 
     /// Get syscalls and BPF functions (if debug symbols are not stripped)
-    fn get_function_symbols(&self) -> BTreeMap<usize, (u32, String)> {
+    pub fn get_function_symbols(&self) -> BTreeMap<usize, (u32, String)> {
         let mut bpf_functions = BTreeMap::new();
         for (hash, (pc, name)) in self.bpf_functions.iter() {
             bpf_functions.insert(*pc, (*hash, name.clone()));
@@ -321,12 +327,10 @@ impl<E: UserDefinedError, I: InstructionMeter> Executable<E, I> for EBpfElf<E, I
     }
 
     /// Get syscalls symbols
-    fn get_syscall_symbols(&self) -> &BTreeMap<u32, String> {
+    pub fn get_syscall_symbols(&self) -> &BTreeMap<u32, String> {
         &self.syscall_symbols
     }
-}
 
-impl<'a, E: UserDefinedError, I: InstructionMeter> EBpfElf<E, I> {
     /// Create from raw text section bytes (list of instructions)
     pub fn new_from_text_bytes(
         config: Config,
@@ -340,7 +344,11 @@ impl<'a, E: UserDefinedError, I: InstructionMeter> EBpfElf<E, I> {
             elf_bytes,
             ro_section: text_bytes.to_vec(),
             text_section_info: SectionInfo {
-                name: ".text".to_string(),
+                name: if config.enable_symbol_and_section_labels {
+                    ".text".to_string()
+                } else {
+                    String::default()
+                },
                 vaddr: ebpf::MM_PROGRAM_START,
                 offset_range: Range {
                     start: 0,
@@ -368,15 +376,21 @@ impl<'a, E: UserDefinedError, I: InstructionMeter> EBpfElf<E, I> {
         // calculate the text section info
         let text_section = Self::get_section(&elf, ".text")?;
         let text_section_info = SectionInfo {
-            name: elf
-                .shdr_strtab
-                .get_at(text_section.sh_name)
-                .unwrap()
-                .to_string(),
+            name: if config.enable_symbol_and_section_labels {
+                elf.shdr_strtab
+                    .get_at(text_section.sh_name)
+                    .unwrap()
+                    .to_string()
+            } else {
+                String::default()
+            },
             vaddr: text_section.sh_addr.saturating_add(ebpf::MM_PROGRAM_START),
             offset_range: text_section.file_range().unwrap_or_default(),
         };
-        if text_section_info.vaddr > ebpf::MM_STACK_START {
+        if (config.reject_section_virtual_address_file_offset_mismatch
+            && text_section.sh_addr != text_section.sh_offset)
+            || text_section_info.vaddr > ebpf::MM_STACK_START
+        {
             return Err(ElfError::ValueOutOfBounds);
         }
 
@@ -399,7 +413,12 @@ impl<'a, E: UserDefinedError, I: InstructionMeter> EBpfElf<E, I> {
         }
         let entrypoint = offset as usize / ebpf::INSN_SIZE;
         bpf_functions.remove(&ebpf::hash_symbol_name(b"entrypoint"));
-        register_bpf_function(&mut bpf_functions, entrypoint, "entrypoint")?;
+        register_bpf_function(
+            &mut bpf_functions,
+            entrypoint,
+            "entrypoint",
+            config.enable_symbol_and_section_labels,
+        )?;
 
         // concatenate the read-only sections into one
         let mut ro_length = text_section.sh_addr as usize + text_section_info.offset_range.len();
@@ -416,7 +435,10 @@ impl<'a, E: UserDefinedError, I: InstructionMeter> EBpfElf<E, I> {
                 let vaddr = section_header
                     .sh_addr
                     .saturating_add(ebpf::MM_PROGRAM_START);
-                if vaddr > ebpf::MM_STACK_START {
+                if (config.reject_section_virtual_address_file_offset_mismatch
+                    && section_header.sh_addr != section_header.sh_offset)
+                    || vaddr > ebpf::MM_STACK_START
+                {
                     return Err(ElfError::ValueOutOfBounds);
                 }
                 let slice = elf_bytes
@@ -456,6 +478,7 @@ impl<'a, E: UserDefinedError, I: InstructionMeter> EBpfElf<E, I> {
 
     /// Fix-ups relative calls
     pub fn fixup_relative_calls(
+        enable_symbol_and_section_labels: bool,
         bpf_functions: &mut BTreeMap<u32, (usize, String)>,
         elf_bytes: &mut [u8],
     ) -> Result<(), ElfError> {
@@ -469,7 +492,12 @@ impl<'a, E: UserDefinedError, I: InstructionMeter> EBpfElf<E, I> {
                     ));
                 }
                 let name = format!("function_{}", target_pc);
-                let hash = register_bpf_function(bpf_functions, target_pc as usize, &name)?;
+                let hash = register_bpf_function(
+                    bpf_functions,
+                    target_pc as usize,
+                    &name,
+                    enable_symbol_and_section_labels,
+                )?;
                 insn.imm = hash as i64;
                 let checked_slice = elf_bytes
                     .get_mut(i * ebpf::INSN_SIZE..(i * ebpf::INSN_SIZE) + ebpf::INSN_SIZE)
@@ -567,6 +595,7 @@ impl<'a, E: UserDefinedError, I: InstructionMeter> EBpfElf<E, I> {
 
         // Fixup all program counter relative call instructions
         Self::fixup_relative_calls(
+            config.enable_symbol_and_section_labels,
             bpf_functions,
             elf_bytes
                 .get_mut(text_section.file_range().unwrap_or_default())
@@ -680,7 +709,12 @@ impl<'a, E: UserDefinedError, I: InstructionMeter> EBpfElf<E, I> {
                         }
                         let target_pc =
                             (sym.st_value - text_section.sh_addr) as usize / ebpf::INSN_SIZE;
-                        register_bpf_function(bpf_functions, target_pc, name)?
+                        register_bpf_function(
+                            bpf_functions,
+                            target_pc,
+                            name,
+                            config.enable_symbol_and_section_labels,
+                        )?
                     } else {
                         // syscall
                         let hash = syscall_cache
@@ -707,29 +741,31 @@ impl<'a, E: UserDefinedError, I: InstructionMeter> EBpfElf<E, I> {
             }
         }
 
-        // Save hashed syscall names for debugging
-        *syscall_symbols = syscall_cache
-            .values()
-            .map(|(hash, name)| (*hash, name.to_string()))
-            .collect();
+        if config.enable_symbol_and_section_labels {
+            // Save syscall names
+            *syscall_symbols = syscall_cache
+                .values()
+                .map(|(hash, name)| (*hash, name.to_string()))
+                .collect();
 
-        // Register all known function names from the symbol table
-        for symbol in &elf.syms {
-            if symbol.st_info & 0xEF != 0x02 {
-                continue;
+            // Register all known function names from the symbol table
+            for symbol in &elf.syms {
+                if symbol.st_info & 0xEF != 0x02 {
+                    continue;
+                }
+                if !text_section
+                    .vm_range()
+                    .contains(&(symbol.st_value as usize))
+                {
+                    return Err(ElfError::ValueOutOfBounds);
+                }
+                let target_pc = (symbol.st_value - text_section.sh_addr) as usize / ebpf::INSN_SIZE;
+                let name = elf
+                    .strtab
+                    .get_at(symbol.st_name)
+                    .ok_or(ElfError::UnknownSymbol(symbol.st_name))?;
+                register_bpf_function(bpf_functions, target_pc, name, true)?;
             }
-            if !text_section
-                .vm_range()
-                .contains(&(symbol.st_value as usize))
-            {
-                return Err(ElfError::ValueOutOfBounds);
-            }
-            let target_pc = (symbol.st_value - text_section.sh_addr) as usize / ebpf::INSN_SIZE;
-            let name = elf
-                .strtab
-                .get_at(symbol.st_name)
-                .ok_or(ElfError::UnknownSymbol(symbol.st_name))?;
-            register_bpf_function(bpf_functions, target_pc, name)?;
         }
 
         Ok(())
@@ -763,7 +799,7 @@ mod test {
     };
     use rand::{distributions::Uniform, Rng};
     use std::{fs::File, io::Read};
-    type ElfExecutable = EBpfElf<UserError, TestInstructionMeter>;
+    type ElfExecutable = Executable<UserError, TestInstructionMeter>;
 
     fn syscall_registry() -> SyscallRegistry {
         let mut syscall_registry = SyscallRegistry::default();
@@ -828,7 +864,7 @@ mod test {
             .expect("validation failed");
         let mut parsed_elf = Elf::parse(&elf_bytes).unwrap();
         let initial_e_entry = parsed_elf.header.e_entry;
-        let executable: &dyn Executable<UserError, TestInstructionMeter> = &elf;
+        let executable: &Executable<UserError, TestInstructionMeter> = &elf;
         assert_eq!(
             0,
             executable
@@ -841,7 +877,7 @@ mod test {
         elf_bytes.pwrite(parsed_elf.header, 0).unwrap();
         let elf = ElfExecutable::load(Config::default(), &elf_bytes, syscall_registry())
             .expect("validation failed");
-        let executable: &dyn Executable<UserError, TestInstructionMeter> = &elf;
+        let executable: &Executable<UserError, TestInstructionMeter> = &elf;
         assert_eq!(
             1,
             executable
@@ -878,7 +914,7 @@ mod test {
         elf_bytes.pwrite(parsed_elf.header, 0).unwrap();
         let elf = ElfExecutable::load(Config::default(), &elf_bytes, syscall_registry())
             .expect("validation failed");
-        let executable: &dyn Executable<UserError, TestInstructionMeter> = &elf;
+        let executable: &Executable<UserError, TestInstructionMeter> = &elf;
         assert_eq!(
             0,
             executable
@@ -900,7 +936,7 @@ mod test {
             0xb7, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
             0x85, 0x10, 0x00, 0x00, 0xfe, 0xff, 0xff, 0xff];
 
-        ElfExecutable::fixup_relative_calls(&mut bpf_functions, &mut prog).unwrap();
+        ElfExecutable::fixup_relative_calls(true, &mut bpf_functions, &mut prog).unwrap();
         let name = "function_4".to_string();
         let hash = hash_bpf_function(4, &name);
         let insn = ebpf::Insn {
@@ -916,7 +952,7 @@ mod test {
         // call +6
         let mut bpf_functions: BTreeMap<u32, (usize, String)> = BTreeMap::new();
         prog.splice(44.., vec![0xfa, 0xff, 0xff, 0xff]);
-        ElfExecutable::fixup_relative_calls(&mut bpf_functions, &mut prog).unwrap();
+        ElfExecutable::fixup_relative_calls(true, &mut bpf_functions, &mut prog).unwrap();
         let name = "function_0".to_string();
         let hash = hash_bpf_function(0, &name);
         let insn = ebpf::Insn {
@@ -943,7 +979,7 @@ mod test {
             0xb7, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
             0xb7, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00];
 
-        ElfExecutable::fixup_relative_calls(&mut bpf_functions, &mut prog).unwrap();
+        ElfExecutable::fixup_relative_calls(true, &mut bpf_functions, &mut prog).unwrap();
         let name = "function_1".to_string();
         let hash = hash_bpf_function(1, &name);
         let insn = ebpf::Insn {
@@ -959,7 +995,7 @@ mod test {
         // call +4
         let mut bpf_functions: BTreeMap<u32, (usize, String)> = BTreeMap::new();
         prog.splice(4..8, vec![0x04, 0x00, 0x00, 0x00]);
-        ElfExecutable::fixup_relative_calls(&mut bpf_functions, &mut prog).unwrap();
+        ElfExecutable::fixup_relative_calls(true, &mut bpf_functions, &mut prog).unwrap();
         let name = "function_5".to_string();
         let hash = hash_bpf_function(5, &name);
         let insn = ebpf::Insn {
@@ -989,7 +1025,7 @@ mod test {
             0xb7, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
             0xb7, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00];
 
-        ElfExecutable::fixup_relative_calls(&mut bpf_functions, &mut prog).unwrap();
+        ElfExecutable::fixup_relative_calls(true, &mut bpf_functions, &mut prog).unwrap();
         let name = "function_1".to_string();
         let hash = hash_bpf_function(1, &name);
         let insn = ebpf::Insn {
@@ -1019,7 +1055,7 @@ mod test {
             0xb7, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
             0x85, 0x10, 0x00, 0x00, 0xf9, 0xff, 0xff, 0xff];
 
-        ElfExecutable::fixup_relative_calls(&mut bpf_functions, &mut prog).unwrap();
+        ElfExecutable::fixup_relative_calls(true, &mut bpf_functions, &mut prog).unwrap();
         let name = "function_4".to_string();
         let hash = hash_bpf_function(4, &name);
         let insn = ebpf::Insn {
