@@ -2,10 +2,10 @@
 
 use crate::disassembler::disassemble_instruction;
 use crate::{
-    ebpf, elf,
+    ebpf,
+    elf::{self, Executable},
     error::UserDefinedError,
-    vm::InstructionMeter,
-    vm::{DynamicAnalysis, Executable},
+    vm::{DynamicAnalysis, InstructionMeter},
 };
 use rustc_demangle::demangle;
 use std::collections::{BTreeMap, BTreeSet, HashMap, HashSet};
@@ -93,7 +93,7 @@ impl Default for CfgNode {
 /// Result of the executable analysis
 pub struct Analysis<'a, E: UserDefinedError, I: InstructionMeter> {
     /// The program which is analyzed
-    pub executable: &'a dyn Executable<E, I>,
+    pub executable: &'a Executable<E, I>,
     /// Plain list of instructions as they occur in the executable
     pub instructions: Vec<ebpf::Insn>,
     /// Functions in the executable
@@ -112,8 +112,8 @@ pub struct Analysis<'a, E: UserDefinedError, I: InstructionMeter> {
 
 impl<'a, E: UserDefinedError, I: InstructionMeter> Analysis<'a, E, I> {
     /// Analyze an executable statically
-    pub fn from_executable(executable: &'a dyn Executable<E, I>) -> Self {
-        let (_program_vm_addr, program) = executable.get_text_bytes().unwrap();
+    pub fn from_executable(executable: &'a Executable<E, I>) -> Self {
+        let (_program_vm_addr, program) = executable.get_text_bytes();
         let functions = executable.get_function_symbols();
         debug_assert!(
             program.len() % ebpf::INSN_SIZE == 0,
